/* $Header: d:/cvsroot/tads/html/win32/mpegamp/mpegamp_w32.h,v 1.2 1999/05/17 02:52:26 MJRoberts Exp $ */

/* Copyright (c) 1998 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  audio_w32.h - win95 audio driver for AMP 0.7.6 MPEG player
Function
  
Notes
  Derived from amp 0.7.6 for use in HTML TADS.
Modified
  10/25/98 MJRoberts  - Creation
*/

#ifndef MPEGAMP_W32_H
#define MPEGAMP_W32_H


#ifndef MPEGAMP_H
#include "mpegamp.h"
#endif

class CMpegAmpW32: public CMpegAmp
{
    friend class CMpegAmp;
    
public:
    CMpegAmpW32(struct IDirectSound *ds);
    ~CMpegAmpW32();

    /* 
     *   Play a file.  This operates synchronously; it doesn't return
     *   until playback finishes.  If 'stop_on_key' is true, we'll cancel
     *   playback if the user presses a key during playback.  
     */
    void play(const char *inFileStr, int stop_on_key);

    /* 
     *   Play from a given file, from the file's current seek position,
     *   consuming a maximum of the given number of bytes from the current
     *   position.  Sends the given message to the given window when
     *   playback is completed.
     *   
     *   This operates asynchronously - this returns as soon as playback
     *   is started, and playback proceeds in the background until
     *   finishes or cancelled via stop().
     *   
     *   Returns zero on success, non-zero on error.  
     */
    int play(const textchar_t *url, HANDLE hfile, DWORD file_size,
             HWND hwnd, UINT done_msg, LPARAM done_msg_lpar);

    /*
     *   Determine if asynchronous playback is still in progress.  Returns
     *   true if playback is still proceeding, false if playback has
     *   finished. 
     */
    int is_playing();

    /* 
     *   Stop playback.  If 'sync' is true, we won't return until we have
     *   confirmed that playback has finished. 
     */
    void stop(int sync);

    /*
     *   Wait until asynchronous playback is finished.  
     */
    void wait_until_done();

private:
    /* low-level playback cancellation */
    void directx_stop_play();

    /* write to the audio device */
    void audioBufferWrite(char *, int);

    /* flush the staging buffer to directx */
    void flush_to_directx();

    /* wait until the playback pointer leaves the last buffer */
    void wait_for_last_buf();

    /* start directx playback */
    void directx_begin_play();

    int  audioBufferOpen(int, int, int);
    void audioBufferClose();
    void audioBufferFlush();
    int  setup_audio(struct AUDIO_HEADER *header);
    int decodeMPEG(void);

    /* playback thread main routine - static entrypoint */
    static void play_thread_main(void *ctx)
    {
        ((CMpegAmpW32 *)ctx)->do_play_thread();
    }

    /* playback thread main routine */
    void do_play_thread();

    /* our DirectSound object */
    IDirectSound *ds_;

    /* our DirectSound buffer object */
    IDirectSoundBuffer *dsbuf_;

    /* wave format descriptor */
    WAVEFORMATEX wf_;

    /* 
     *   Size of each block in our direct sound buffer.  The underlying
     *   buffer consists of three blocks of this size. 
     */
    DWORD data_size_;

    /* offset in the direct sound buffer of next write position */
    DWORD write_csr_;

    /* current locked direct sound buffer blocks */
    void *lock_ptr1_, *lock_ptr2_;
    DWORD lock_len1_, lock_len2_;

    /* the next available byte in the locked buffer block */
    char *stage_buf_cur_;

    /* the end of the locked buffer block */
    char *stage_buf_end_;

    /* flag indicating whether the underlying directx buffer is playing */
    int playing_;

    /* 
     *   flag indicating whether or not we ever started playback - this
     *   allows us to distinguish the case that we reach the end of a short
     *   file without every having started buffered playback from the case
     *   where we cancel playback at some point after starting it 
     */
    int started_playing_;

    /* number of buffers we've filled and flushed to directx */
    int flush_count_;

    /* 
     *   done notification window -- when we finish playback, we'll notify
     *   the given window via the given message and lparam
     */
    HWND done_hwnd_;
    UINT done_msg_;
    LPARAM done_msg_lpar_;

    /* the thread that's handling playback, if applicable */
    HANDLE thread_hdl_;
    DWORD thread_id_;

    /* the URL of the sound resource (used only for displaying errors) */
    const char *url_;
};

#endif /* AUDIO_W95_H */

