#ifdef RCSID
static char RCSid[] =
"$Header: d:/cvsroot/tads/html/win32/tadspng.cpp,v 1.2 1999/05/17 02:52:25 MJRoberts Exp $";
#endif

/* Copyright (c) 1997 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  tadspng.cpp - Win32 PNG object implementation
Function
  
Notes
  
Modified
  11/08/97 MJRoberts  - Creation
*/

#include <windows.h>

#ifndef TADSHTML_H
#include "tadshtml.h"
#endif
#ifndef TADSWIN_H
#include "tadswin.h"
#endif
#ifndef HTMLPNG_H
#include "htmlpng.h"
#endif
#ifndef TADSPNG_H
#include "tadspng.h"
#endif


CTadsPng::CTadsPng()
{
}

CTadsPng::~CTadsPng()
{
}

/*
 *   Load the iamge from a PNG object 
 */
int CTadsPng::load_from_png(CHtmlPng *png)
{
    int pix_bytes_per_pixel;
    
    /* drop any previously loaded image */
    delete_image();

    /* create a logical palette from the PNG's palette, if it has one */
    create_palhdl(png->get_palette(), png->get_palette_size());

    /* create a DWORD-aligned pixel buffer from the PNG rows */
    if (create_pix_dword_aligned(png->get_rows(),
                                 png->get_width(),
                                 png->get_row_bytes(),
                                 png->get_height(),
                                 &pix_bytes_per_pixel))
        return 1;

    /* if there's alpha information, create the mask */
    if (png->get_bits_per_pixel() == 32 || png->get_alpha() != 0)
    {
        /* create the DWORD-aligned pixel buffer for the mask */
        if (create_mask(png, pix_bytes_per_pixel))
            return 1;
    }

    /* success */
    return 0;
}

/*
 *   Create a DWORD-aligned pixel buffer from an array of PNG rows 
 */
int CTadsPng::create_pix_dword_aligned(const png_byte *const *src_rows,
                                       unsigned long width_pix,
                                       unsigned long width_bytes,
                                       unsigned long height,
                                       int *pix_bytes_per_pixel)
{
    long out_width_bytes;
    long in_width_bytes;
    long width_bits;
    long new_size;
    unsigned long row;
    OS_HUGEPTR(unsigned char) dst;
    const png_byte *const *src;
    int in_bytes_per_pixel;
    int out_bytes_per_pixel;

    /* store the width and height */
    width_ = width_pix;
    height_ = height;

    /* if there are no rows, we can't do anything */
    if (src_rows == 0)
        return 1;

    /* figure out how many bytes per pixel */
    in_width_bytes = width_bytes;
    in_bytes_per_pixel = in_width_bytes / width_pix;
    out_bytes_per_pixel = (in_bytes_per_pixel > 3 ? 3 : in_bytes_per_pixel);

    /* tell the caller the bytes per pixel in the output image */
    *pix_bytes_per_pixel = out_bytes_per_pixel;

    /* remember the bit depth of the bitmap data we're storing */
    bpp_ = out_bytes_per_pixel * 8;

    /* calculate the smallest DWORD-aligned buffer we can use */
    width_bits = width_pix * out_bytes_per_pixel * 8;
    out_width_bytes = ((width_bits + 31) / 32) * 4;

    /* calculate the size we'll need for the DWORD-aligned buffer */
    new_size = (unsigned long)out_width_bytes * (unsigned long)height;

    /* allocate our buffer */
    pix_ = (OS_HUGEPTR(unsigned char))os_alloc_huge(new_size);
    if (pix_ == 0)
        return 1;

    /* copy the rows */
    dst = os_add_huge(pix_, out_width_bytes * (height - 1));
    if (in_bytes_per_pixel == out_bytes_per_pixel)
    {
        /* the pixels are the same size, so we can copy a row at a time */
        for (src = src_rows, row = 0 ; row < height ; ++row, ++src)
        {
            /* copy this row */
            memcpy(dst, *src, in_bytes_per_pixel * width_pix);

            /* move to next row of output */
            dst = os_add_huge(dst, -out_width_bytes);
        }
    }
    else
    {
        /* 
         *   we need to adjust the size of each pixel, so copy only one
         *   pixel at a time 
         */
        for (src = src_rows, row = 0 ; row < height ; ++row, ++src)
        {
            unsigned long i;
            const png_byte *srcp;
            unsigned char *dstp;
            
            /* copy this row */
            for (i = 0, srcp = *src, dstp = dst ; i < width_pix ;
                 ++i, srcp += in_bytes_per_pixel, dstp += out_bytes_per_pixel)
            {
                /* copy this pixel */
                memcpy(dstp, srcp, out_bytes_per_pixel);
            }
            
            /* move to next row */
            dst = os_add_huge(dst, -out_width_bytes);
        }
    }

    /* remember the byte width */
    width_bytes_ = out_width_bytes;

    /* success */
    return 0;
}

/*
 *   Create the mask, if we have transparency information 
 */
int CTadsPng::create_mask(CHtmlPng *png, int pix_bytes_per_pixel)
{
    long out_width_bytes;
    long pix_width_bits;
    long pix_width_bytes;
    long width_bits;
    long new_size;
    unsigned long row;
    OS_HUGEPTR(unsigned char) dst;
    OS_HUGEPTR(unsigned char) pix;
    const png_byte *const *src;
    size_t srclen;
    int black_idx;

    /* 
     *   if this is a paletted (8-bit) image, find the 'black' color in
     *   the palette 
     */
    black_idx = 0;
    if (pix_bytes_per_pixel == 1)
    {
        size_t i;
        HTML_color_t *pal;
        
        /* search the palette for black */
        for (i = 0, pal = png->get_palette() ;
             i < png->get_palette_size() ; ++i, ++pal)
        {
            if (*pal == HTML_make_color(0, 0, 0))
            {
                /* this is the one */
                black_idx = i;
                break;
            }
        }
    }

    /* 
     *   calculate the smallest DWORD-aligned buffer we can use - we only
     *   need to store a monochrome bitmap, so we need one byte per 8
     *   pixels 
     */
    width_bits = width_;
    out_width_bytes = ((width_bits + 31) / 32) * 4;

    /* calculate the width of each row of the pix_ array */
    pix_width_bits = pix_bytes_per_pixel * 8 * width_;
    pix_width_bytes = ((pix_width_bits + 31) / 32) * 4;

    /* calculate the size we'll need for the DWORD-aligned buffer */
    new_size = (unsigned long)out_width_bytes * (unsigned long)height_;

    /* allocate our buffer */
    mask_ = (OS_HUGEPTR(unsigned char))os_alloc_huge(new_size);
    if (mask_ == 0)
        return 1;

    /* read either from the separate alpha channel or the mixed-in alpha */
    if (png->get_alpha() != 0)
    {
        /* read from the separate one-byte-per-pixel alpha channel */
        src = png->get_alpha();
        srclen = 1;
    }
    else
    {
        /* read from the alpha channel mixed into the RGBA data */
        src = png->get_rows();
        srclen = 4;
    }

    /* 
     *   initialize the entire mask to 1 bits - we want to leave the mask
     *   as 1 wherever the alpha channel indicates transparency, since we
     *   AND the mask with the background to leave only the parts of the
     *   background where the image is transparent 
     */
    for (row = 0, dst = mask_ ; row < height_ ;
         ++row, dst = os_add_huge(dst, out_width_bytes))
    {
        /* set this row to all 1's */
        memset(dst, 0xff, out_width_bytes);
    }

    /* 
     *   set up a pointer to the image, so we can clear the transparent
     *   pixels to black 
     */
    pix = os_add_huge(pix_, pix_width_bytes * (height_ - 1));

    /* copy the rows */
    dst = os_add_huge(mask_, out_width_bytes * (height_ - 1));
    for (row = 0 ; row < height_ ; ++row, ++src)
    {
        unsigned long i;
        const png_byte *srcp;
        unsigned char *pixp;
        unsigned char *dstp;
        int bitpos;

        /* 
         *   start out at the first alpha byte in this row - if we're
         *   reading from RGBA data, note that the first A byte is at
         *   offset 3 in the row (after the R, G, and B bytes) 
         */
        srcp = *src + (srclen - 1);
        
        /* copy this row */
        for (i = 0, dstp = dst, pixp = pix, bitpos = 0 ; i < width_ ;
             ++i, srcp += srclen, pixp += pix_bytes_per_pixel)
        {
            /* 
             *   If this pixel is not transparent, clear this bit of the
             *   mask - this will ensure that we clear the background for
             *   each non-transparent pixel of the image. 
             */
            if (*srcp != 0)
            {
                /* non-transparent - clear the mask here */
                *dstp &= ~(1 << (7 - bitpos));
            }
            else
            {
                /* transparent - clear the image here */
                memset(pixp, black_idx, pix_bytes_per_pixel);
            }

            /* advance to the next bit position */
            ++bitpos;

            /* if bitpos has reached 8, move to the next byte */
            dstp += ((bitpos & 0x08) >> 3);

            /* when bitpos reaches 8, wrap to 0 */
            bitpos &= 0x07;
        }
        
        /* move to next row of the mask */
        dst = os_add_huge(dst, -out_width_bytes);

        /* move to the next row of the picture */
        pix = os_add_huge(pix, -pix_width_bytes);
    }

    /* success */
    return 0;
}

