#ifdef RCSID
static char RCSid[] =
"$Header: d:/cvsroot/tads/html/win32/w32img.cpp,v 1.2 1999/05/17 02:52:26 MJRoberts Exp $";
#endif

/* Copyright (c) 1998 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  w32img.cpp - tads html win32 image implementation
Function
  
Notes
  
Modified
  01/31/98 MJRoberts  - Creation
*/

#include <Windows.h>

#ifndef TADSHTML_H
#include "tadshtml.h"
#endif
#ifndef HTMLURL_H
#include "htmlurl.h"
#endif
#ifndef HTMLSYS_H
#include "htmlsys.h"
#endif
#ifndef HTMLRC_H
#include "htmlrc.h"
#endif
#ifndef HTMLJPEG_H
#include "htmljpeg.h"
#endif
#ifndef HTMLPNG_H
#include "htmlpng.h"
#endif
#ifndef TADSJPEG_H
#include "tadsjpeg.h"
#endif
#ifndef TADSPNG_H
#include "tadspng.h"
#endif
#ifndef W32IMG_H
#include "w32img.h"
#endif
#ifndef HTMLW32_H
#include "htmlw32.h"
#endif


/* ------------------------------------------------------------------------ */
/*
 *   JPEG Image implementation 
 */

/* create a new system-specific JPEG display object */
CHtmlSysResource *CHtmlSysImageJpeg::
   create_jpeg(const CHtmlUrl *url, const textchar_t *filename,
               unsigned long seekpos, unsigned long filesize,
               CHtmlSysWin *win)
{
    CHtmlJpeg *jpeg;
    CHtmlSysImageJpeg_win32 *img = 0;
    int err;

    /* create the helper object to load the data */
    jpeg = new CHtmlJpeg();

    /* load the data from the file */
    err = jpeg->read_jpeg_file(filename, seekpos, filesize,
                               win->get_use_palette());
    if (err != 0)
    {
        oshtml_dbg_printf("jpeg: error %d loading image %s\n",
                          err, url->get_url());
    }
    else
    {
        /* 
         *   successfully loaded - create the jpeg display object using
         *   the data loaded into the helper
         */
        img = new CHtmlSysImageJpeg_win32();
        if ((err = img->load_from_jpeg(jpeg)) != 0)
        {
            /* failure - delete it and log the error */
            delete img;
            img = 0;
            oshtml_dbg_printf("jpeg: error %d preparing image %s\n",
                              err, url->get_url());
        }
    }

    /* we're done with the helper object - delete it */
    delete jpeg;

    /* return the result */
    return img;
}

/* map the palette */
int CHtmlSysImageJpeg_win32::map_palette(CHtmlSysWin *win, int foreground)
{
    return CTadsJpeg::map_palette((CHtmlSysWin_win32 *)win, foreground);
}

/*
 *   Draw the image 
 */
void CHtmlSysImageJpeg_win32::draw_image(CHtmlSysWin *syswin,
                                         CHtmlRect *docpos)
{
    CHtmlSysWin_win32 *win = (CHtmlSysWin_win32 *)syswin;
    CHtmlRect winpos = win->doc_to_screen(*docpos);

    CTadsJpeg::draw(win, &winpos, TADSIMG_DRAW_STRETCH);
}


/* ------------------------------------------------------------------------ */
/*
 *   PNG Image implementation 
 */

/* create a new system-specific PNG display object */
CHtmlSysResource *CHtmlSysImagePng::
   create_png(const CHtmlUrl *url, const textchar_t *filename,
              unsigned long seekpos, unsigned long filesize,
              CHtmlSysWin *win)
{
    CHtmlPng *png;
    CHtmlSysImagePng_win32 *img = 0;
    int err;

    /* load the PNG from the file, using the helper class */
    png = new CHtmlPng();
    err = png->read_png_file(filename, seekpos, filesize,
                             win->get_use_palette());
    if (err != 0)
    {
        oshtml_dbg_printf("png: error %d loading image %s\n",
                          err, url->get_url());
    }
    else
    {
        /* 
         *   successfully loaded - create a PNG display object using the
         *   data we've loaded into the helper object 
         */
        img = new CHtmlSysImagePng_win32();
        if ((err = img->load_from_png(png)) != 0)
        {
            /* failure - delete it and log the error */
            delete img;
            img = 0;
            oshtml_dbg_printf("png: error %d preparing image %s\n",
                              err, url->get_url());
        }
    }

    /* done with the png loader helper object - delete it */
    delete png;

    /* return the image */
    return img;
}

/* map the palette */
int CHtmlSysImagePng_win32::map_palette(CHtmlSysWin *win, int foreground)
{
    return CTadsPng::map_palette((CHtmlSysWin_win32 *)win, foreground);
}

/*
 *   Draw the image 
 */
void CHtmlSysImagePng_win32::draw_image(CHtmlSysWin *syswin,
                                        CHtmlRect *docpos)
{
    CHtmlSysWin_win32 *win = (CHtmlSysWin_win32 *)syswin;
    CHtmlRect winpos = win->doc_to_screen(*docpos);

    CTadsPng::draw(win, &winpos, TADSIMG_DRAW_STRETCH);
}


